@@ -60,23 +60,23 @@ use std::sync::Arc;
 ///
 /// ```text
 ///                                 ┌ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┐
-///                                                         ┌ ─ ─ ─ ─ ─ ─ ─ ─┐    
+///                                                         ┌ ─ ─ ─ ─ ─ ─ ─ ─┐
 ///  ┌─────────────┐                │     ┌───┐               ┌───┐ ┌──────┐      │
-///  │   [A,B]     │                      │ 1 │             │ │ 1 │ │  A   │ │ 0  
+///  │   [A,B]     │                      │ 1 │             │ │ 1 │ │  A   │ │ 0
 ///  ├─────────────┤                │     ├───┤               ├───┤ ├──────┤      │
-///  │    NULL     │                      │ 0 │             │ │ 1 │ │  B   │ │ 1  
+///  │    NULL     │                      │ 0 │             │ │ 1 │ │  B   │ │ 1
 ///  ├─────────────┤                │     ├───┤               ├───┤ ├──────┤      │
-///  │  [C,NULL]   │                      │ 1 │             │ │ 0 │ │ ???? │ │ 2  
+///  │  [C,NULL]   │                      │ 1 │             │ │ 0 │ │ ???? │ │ 2
 ///  └─────────────┘                │     └───┘               ├───┤ ├──────┤      │
-///                                                         | │ 0 │ │ ???? │ │ 3  
+///                                                         | │ 0 │ │ ???? │ │ 3
 ///  Logical Values                 │   Validity              ├───┤ ├──────┤      │
-///                                     (nulls)             │ │ 1 │ │  C   │ │ 4  
+///                                     (nulls)             │ │ 1 │ │  C   │ │ 4
 ///                                 │                         ├───┤ ├──────┤      │
-///                                                         │ │ 0 │ │ ???? │ │ 5  
+///                                                         │ │ 0 │ │ ???? │ │ 5
 ///                                 │                         └───┘ └──────┘      │
-///                                                         │     Values     │    
+///                                                         │     Values     │
 ///                                 │   FixedSizeListArray        (Array)         │
-///                                                         └ ─ ─ ─ ─ ─ ─ ─ ─┘    
+///                                                         └ ─ ─ ─ ─ ─ ─ ─ ─┘
 ///                                 └ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘
 /// ```
 ///
@@ -448,7 +448,7 @@ impl std::fmt::Debug for FixedSizeListArray {
     }
 }
 
-impl<'a> ArrayAccessor for &'a FixedSizeListArray {
+impl ArrayAccessor for &FixedSizeListArray {
     type Item = ArrayRef;
 
     fn value(&self, index: usize) -> Self::Item {
